var NS_VCARD = 'vcard-temp';
var NS_CAPS= 'http://jabber.org/protocol/caps';
var NS_PEP = 'http://jabber.org/protocol/pubsub#event';
var NS_TUNE = 'http://jabber.org/protocol/tune';
var BOSH_SERVICE = 'http://localhost:5280/http-bind/';

var connection   = null;
var show_log     = true;

var features = new Array(NS_CAPS, NS_TUNE+'+notify', Strophe.NS.DISCO_INFO);
var appName = 'socialXMPP';

var roster = new Array();

function log(msg) {
    var entry = $('<div></div>').append(Strophe.xmlTextNode(msg));
    $('#log').append(entry);
}

function rawInput(data) {
    log('RECV: ' + data);
}

function rawOutput(data) {
    log('SENT: ' + data);
}

function jid2id(jid) {
    jid = Strophe.getBareJidFromJid(jid);
    return jid.split('@').join('-').split('.').join('-');
}

function populateVCard(e, jid) {
    var easy_cases = new Array('FN', 'FAMILY', 'MIDDLE', 'GIVEN', 'NICKNAME', 'BDAY', 'CTRY', 'USERID');
    for (var i=0; i<easy_cases.length; i++) {
	var text = e.getElementsByTagName(easy_cases[i])[0];
	if (text) {
	    text = Strophe.getText(text);
	    $('#'+easy_cases[i]).append(Strophe.xmlTextNode(text));
	}
    }
    var avatar = e.getElementsByTagName('PHOTO')[0];
    if (avatar) {
	var mime = Strophe.getText(avatar.getElementsByTagName('TYPE')[0]);
	var binval = Strophe.getText(avatar.getElementsByTagName('BINVAL')[0]);

	$("#"+jid2id(jid)+" img").attr('src', 'data:'+mime+';base64,'+binval);
	$("#PHOTO img").attr('src', 'data:'+mime+';base64,'+binval);
    }
    $('#vCard_container').slideDown("normal");
}

function _cbVCard(e) {
    var jid = e.getAttribute('from');
    if (roster[jid2id(jid)].vCard == "") {
	roster[jid2id(jid)].vCard = e;
    }
    if ($('#vCard_container').is(':visible')) {
	$('#vCard_container').hide();
    }
    $('#vCard_container').empty();
    $('#vCard_container').load('vCard.html #vCard', function() {populateVCard(e, jid);});

    return false;
}

function getVCard(jid) {
    var id = 'getvCard'+jid2id(jid);
    if (roster[jid2id(jid)].vCard == "") {
	var vCardiq = $iq({'to':jid,
			'id':id,
			'type':'get'}
		).c('vCard', {'xmlns':NS_VCARD});
	connection.addHandler(_cbVCard, null, 'iq', 'result', id);
	connection.send(vCardiq.tree());
    } else {
	_cbVCard(roster[jid2id(jid)].vCard);
    }

}

function addFriend(jid, nick) {
    roster[jid2id(jid)] = new Buddy(nick, jid);
    $('#friends').append('<div class="friend" id="'+jid2id(jid)+'"><img src="imgs/none.png" /><br /><span class="nick">'+nick+'</span></div>');
    $('#'+jid2id(jid)).click(function() {
    	getVCard(jid);
    });
}

function _cbRoster(e) {
    var query = e.getElementsByTagName('query')[0];
    var entries = query.getElementsByTagName('item');
    for (var item=0; item<entries.length; item++) {
	nick = entries[item].getAttribute('name');
	if (!nick) {
	    nick = entries[item].getAttribute('jid').split('@')[0];
	}
	addFriend(entries[item].getAttribute('jid'), nick);
    }
    connection.addHandler(_cbPEP, NS_PEP, 'message');
    var initialPresence = $pres().c('c', {xmlns: NS_CAPS, hash: 'sha-1', node: 'http://jabber.babelmonkeys.de', ver: genCaps()}).up();
    connection.send(initialPresence.tree());

    return false;
}

function getRoster() {
    var id = 'getRoster';

    var rosteriq = $iq({'id':id,
			'type':'get'}
	).c('query', {'xmlns':Strophe.NS.ROSTER});

    connection.addHandler(_cbRoster, null, 'iq', 'result', id);
    connection.send(rosteriq.tree());
}

function _cbDisco(e) {
    var id = e.getAttribute('id');
    var jid = e.getAttribute('from');

    var response = $iq({id: id, type: 'result', to: jid});
    var query = response.c('query', {xmlns: Strophe.NS.DISCO_INFO})
    query.c('identity', {category: 'client', type: 'web', name: appName}).up();
    for (var i = 0; i < features.length; i++) {
	query.c('feature', {'var': features[i]}).up();
    }
    connection.send(response.tree());

    return true;
}

function genCaps() {
    var S = '';
    S += 'client/web//' + appName + '<'
    features.sort();
    for (var i = 0; i < features.length; i++) {
	S += features[i] + '<';
    }
    return b64_sha1(S);
}

function _cbPEP(e) {
    var from = e.getAttribute('from');
    if (Strophe.getBareJidFromJid(from) == Strophe.getBareJidFromJid(connection.jid)) {
	return true; // Drop own PEP events
    }
    var items = e.getElementsByTagName('items')[0];
    // Handle Tune
    if (items.getAttribute('node') == NS_TUNE) {
	var tune = new Tune();
	if (items.getElementsByTagName('tune')[0].childNodes.length > 0) {
	    tune.artist = Strophe.getText(items.getElementsByTagName('artist')[0]);
	    tune.length= Strophe.getText(items.getElementsByTagName('length')[0]);
	    tune.rating= Strophe.getText(items.getElementsByTagName('rating')[0]);
	    tune.source= Strophe.getText(items.getElementsByTagName('source')[0]);
	    tune.title= Strophe.getText(items.getElementsByTagName('title')[0]);
	    tune.track= Strophe.getText(items.getElementsByTagName('track')[0]);
	    tune.uri= Strophe.getText(items.getElementsByTagName('uri')[0]);
	    if ( $('#'+jid2id(from)+' .tune').length > 0 ) {
		$('#'+jid2id(from)+' .tune').empty();
		$('#'+jid2id(from)+' .tune').append(Strophe.xmlTextNode('Listening to '+ tune.title + ' by ' + tune.artist + ' from ' + tune.source));
	    } else {
		$('#'+jid2id(from)).append('<br /><span class="tune">Listening to '+ tune.title + ' by ' + tune.artist + ' from ' + tune.source + '</span>');
	    }
	} else {
	    if ( $('#'+jid2id(from)+' .tune').length > 0 ) {
		$('#'+jid2id(from)+' .tune').empty();
	    }
	}
	roster[jid2id(from)].tune = tune;
    }
    return true;
}

function onConnect(status) {
    if (status == Strophe.Status.CONNECTING) {
	log('Strophe is connecting.');

    } else if (status == Strophe.Status.CONNFAIL) {
	log('Strophe failed to connect.');
	showConnect();
    } else if (status == Strophe.Status.DISCONNECTING) {
	log('Strophe is disconnecting.');
    } else if (status == Strophe.Status.DISCONNECTED) {
	log('Strophe is disconnected.');
	showConnect();

    } else if (status == Strophe.Status.CONNECTED) {
	log('Strophe is connected.');
	getRoster();
	connection.addHandler(_cbDisco, Strophe.NS.DISCO_INFO, 'iq', 'get');
    }
}

function showConnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);	

    $('#log').empty();
    $('#vCard_container').empty();
    $('#friends').empty();
    $('label').show();
    jid.show();
    pass.show();
    button.value = 'connect';
    return false;
}

function showDisconnect() {
    var jid = $('#jid');
    var pass = $('#pass');
    var button = $('#connect').get(0);	

    button.value = 'disconnect';
    pass.hide();
    jid.hide();
    $('label').hide();
    return false;
}

$(document).ready(function () {
    connection = new Strophe.Connection(BOSH_SERVICE);
    connection.rawInput = rawInput;
    connection.rawOutput = rawOutput;

    $("#log_toggle").click(function () {
	$("#log").toggle();	
      });
      
    $('#vCard_container').click(function () {
        $('#vCard_container').slideUp("normal");
      });

    $('#cred').bind('submit', function () {
	var button = $('#connect').get(0);
	var jid = $('#jid');
	var pass = $('#pass');	
	
	if (button.value == 'connect') {
 	    showDisconnect();
	    connection.connect(jid.get(0).value,
			       pass.get(0).value,
			       onConnect);
	} else {
	    connection.disconnect();
	    showConnect();
	}
	return false;
    });
});
